/******************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence
{
    using System;
    using System.Reflection;
    using System.Collections;
    using System.Collections.Generic;



    /// <summary> Used to override a mapping for an entity relationship.
    /// <para>
    ///  May be applied to an entity that extends a mapped superclass to
    /// override a relationship mapping defined by the mapped
    /// superclass. If not specified, the association is mapped the same as
    /// in the original mapping. When used to override a mapping defined by
    /// a mapped superclass, <c>AssociationOverride</c> is applied to
    /// the entity class.
    /// </para>
    /// <para>
    ///  May be used to override a relationship mapping from an
    /// embeddable within an entity to another entity when the embeddable
    /// is on the owning side of the relationship. When used to override a
    /// relationship mapping defined by an embeddable class (including an
    /// embeddable class embedded within another embeddable class),
    /// <c>AssociationOverride</c> is applied to the field or
    /// property containing the embeddable.
    /// </para>
    /// <para>
    ///  When <c>AssociationOverride</c> is used to override a
    /// relationship mapping from an embeddable class, the
    /// <c>name</c> element specifies the referencing relationship
    /// field or property within the embeddable class. To override mappings
    /// at multiple levels of embedding, a dot (".") notation syntax must
    /// be used in the <c>name</c> element to indicate an attribute
    /// within an embedded attribute.  The value of each identifier used
    /// with the dot notation is the name of the respective embedded field
    /// or property.
    /// </para>
    /// <para>
    ///  When <c>AssociationOverride</c> is applied to override
    /// the mappings of an embeddable class used as a map value,
    /// "<c>value.</c>" must be used to prefix the name of the
    /// attribute within the embeddable class that is being overridden in
    /// order to specify it as part of the map value.
    /// </para>
    /// <para>
    ///  If the relationship mapping is a foreign key mapping, the
    /// <c>joinColumn</c> elements are used.  If the relationship
    /// mapping uses a join table, the <c>joinTable</c> element must
    /// be specified to override the mapping of the join table and/or its
    /// join columns.
    /// </para>
    /// 
    /// <example><code>
    ///    Example 1: Overriding the mapping of a relationship defined by a mapped superclass
    /// 
    ///    &#91;MappedSuperclass&#93;
    ///    public class Employee {
    ///        ...
    ///        &#91;ManyToOne&#93;
    ///        protected Address address;
    ///        ...
    ///    }
    ///    
    ///    &#91;Entity&#93; 
    ///        &#91;AssociationOverride(Name="address")&#93;
    ///        &#91;JoinColumn(AttibuteRef="address",Name="ADDR_ID")&#93;
    ///        // address field mapping overridden to ADDR_ID foreign key
    ///    public class PartTimeEmployee extends Employee {
    ///        ...
    ///    }
    /// </code></example>
    /// 
    /// <example><code>
    ///    Example 2: Overriding the mapping for phoneNumbers defined in the ContactInfo class
    /// 
    ///      [Entity]
    ///     public class Employee {
    ///         [Id]
    ///         public virtual int id { get; set; }
    ///         [AssociationOverride(Name=&quot;phoneNumbers&quot;)]
    ///         [JoinTable(Name=&quot;EMPPHONES&quot;, AssociationOverrideRef=&quot;phoneNumbers&quot;)]
    ///         [JoinColumn(AttributeRef=&quot;EMPPHONES&quot;, Name=&quot;EMP&quot;)]
    ///         [JoinColumn(AttributeRef=&quot;EMPPHONES&quot;,Name=&quot;PHONE&quot;, Inverse=true)]
    ///         [Embedded]
    ///         public virtual ContactInfo contactInfo { get; set; }
    ///     }
    /// 
    ///     [Embeddable]
    ///     public class ContactInfo {
    ///         [ManyToOne] Address address; // Unidirectional
    ///         [ManyToMany(TargetEntity=typeof(PhoneNumber))]
    ///         public virtual IList phoneNumbers { get; set; }
    ///      }
    /// 
    ///    [Entity]
    ///     public class PhoneNumber {
    ///         [Id]
    ///         public virtual int number { get; set; }
    ///         [ManyToMany(MappedBy=&quot;contactInfo.phoneNumbers&quot;)]
    ///         public virtual Collection&lt;Employee&gt; employees { get; set; }
    ///     }
    ///    </code></example>
    /// 
    /// <seealso cref="Embedded"/>
    /// <seealso cref="Embeddable"/>
    /// <seealso cref="MappedSuperclass"/>
    /// <seealso cref="AttributeOverride"/>
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Class | System.AttributeTargets.Field | System.AttributeTargets.Property, AllowMultiple = true)]


    public sealed class AssociationOverride : System.Attribute
    {

        /// <summary> (Required) The name of the relationship property whose mapping is
        /// being overridden if property-based access is being used,
        /// or the name of the relationship field if field-based access is used.
        /// </summary>
        public string Name;
    }
}