/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;


    /// <summary> Specifies the table that is used for the mapping of
    /// collections of basic or embeddable types.  Applied
    /// to the collection-valued field or property.
    ///<para>
    /// By default, the columns of the collection table that correspond
    /// to the embeddable class or basic type are derived from the
    /// attributes of the embeddable class or from the basic type according
    /// to the default values of the <c>Column</c> annotation. In the case
    /// of a basic type, the column name is derived from the name of the
    /// collection-valued field or property. In the case of an embeddable
    /// class, the column names are derived from the field or property
    /// names of the embeddable class.
    /// </para>
    /// <ul>
    /// <li> To override the default properties of the column used for a
    /// basic type, the <c>Column</c> annotation is used on the
    /// collection-valued attribute in addition to the
    /// <c>ElementCollection</c> annotation. 
    /// </li>
    /// <li> To override these defaults for an embeddable class, the
    /// <c>AttributeOverride</c>  annotations can be used in
    /// addition to the <c>ElementCollection</c> annotation. If the
    /// embeddable class contains references to other entities, the default
    /// values for the columns corresponding to those references may be
    /// overridden by means of the <c>AssociationOverride</c> annotations.  
    /// </li>
    /// </ul>
    /// 
    ///  If the <c>CollectionTable</c> annotation is missing, the
    /// default values of the <c>CollectionTable</c> annotation
    /// elements apply.
    /// 
    /// <example><code>
    ///    Example:
    /// 
    ///    &#91;Embeddable&#93; public class Address {
    ///       protected String street;
    ///       protected String city;
    ///       protected String state;
    ///       ... 
    ///     }
    /// 
    ///    &#91;Entity&#93; public class Person {
    ///       &#91;Id&#93; protected String ssn;
    ///       protected String name;
    ///       protected Address home;
    ///       ...
    ///       &#91;ElementCollection&#93;  // use default table (PERSON_NICKNAMES)
    ///       &#91;Column(name="name", length=50)&#93;
    ///       protected Set&#060;String&#062; nickNames = new HashSet();
    ///       ...
    ///    }
    /// 
    ///    &#91;Entity&#93; public class WealthyPerson extends Person {
    ///       &#91;ElementCollection&#93;
    ///       &#91;CollectionTable(name="HOMES")&#93; // use default join column name
    ///          &#91;AttributeOverride(PropertyName="street", Name="HOME_STREET")&#93;
    ///          &#91;AttributeOverride(PropertyName="city", Name="HOME_CITY")&#93;
    ///          &#91;AttributeOverride(PropertyName="state", Name="HOME_STATE")&#93;
    ///          &#91;UniqueConstraint(TableRef = "HOMES", Name = "unique1", ColumnNames = new String[] { "HOME_STREET", "HOME_CITY" })&#93;
    ///       protected Set&#060;Address&#062; vacationHomes = new HashSet();
    ///       ...
    ///    }
    /// </code></example>
    /// 
    /// <seealso cref="ElementCollection"/>
    /// <seealso cref="AttributeOverride"/>
    /// <seealso cref="AssociationOverride"/>
    /// <seealso cref="Column"/>
    /// 
    /// 
    /// </summary>

    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property)]/*@Target( { METHOD, FIELD })*/

public sealed class CollectionTable : System.Attribute {

    /// <summary> 
    ///  (Optional) The name of the collection table.  If not specified,
    ///  it defaults to the concatenation of the name of the containing
    ///  entity and the name of the collection attribute, separated by
    ///  an underscore.
    /// </summary>
    public string Name = "";

    /// <summary>  (Optional) The catalog of the table.  If not specified, the
    ///  default catalog is used.
    /// </summary>
    public string Catalog = "";

    /// <summary> (Optional) The schema of the table.  If not specified, the
    /// default schema for the user is used.
    /// </summary>
    public string Schema = "";

}
}