/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;


    /// <summary> Is used to specify the mapped column for a persistent property or field.
    /// If no <c>Column</c> annotation is specified, the default values apply.
    /// 
    /// <blockquote>
    /// 
    /// <example><code>
    ///    Example 1:
    /// 
    ///    &#91;Column(name="DESC", nullable=false, length=512)&#93;
    ///    public String getDescription() { return description; }
    /// 
    ///    Example 2:
    /// 
    ///    &#91;Column(name="DESC",
    ///            columnDefinition="CLOB NOT NULL",
    ///            table="EMP_DETAIL")&#93;
    ///    &#91;Lob&#93;
    ///    public String getDescription() { return description; }
    /// 
    ///    Example 3:
    /// 
    ///    &#91;Column(name="ORDER_COST", updatable=false, precision=12, scale=2) &#93;
    ///    public BigDecimal getCost() { return cost; }
    /// 
    /// </code></example>
    /// </blockquote>
    /// 
    /// 
    /// 
    /// </summary>
    [System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property)] 

public class Column : System.Attribute {

    /// <summary> (Optional) The name of the column. Defaults to 
    /// the property or field name.
    /// </summary>
    public string Name = "";

    /// <summary> (Optional) Whether the column is a unique key.  This is a 
    /// shortcut for the <c>UniqueConstraint</c> annotation at the table 
    /// level and is useful for when the unique key constraint 
    /// corresponds to only a single column. This constraint applies 
    /// in addition to any constraint entailed by primary key mapping and 
    /// to constraints specified at the table level.
    /// </summary>
    public bool Unique = false;

    /// <summary> (Optional) Whether the database column is nullable.
    /// </summary>
    public bool Nullable = true;

    /// <summary> (Optional) Whether the column is included in SQL INSERT 
    /// statements generated by the persistence provider.
    /// </summary>
    public bool Insertable = true;

    /// <summary> (Optional) Whether the column is included in SQL UPDATE 
    /// statements generated by the persistence provider.
    /// </summary>
    public bool Updatable = true;

    /// <summary> (Optional) The SQL fragment that is used when 
    /// generating the DDL for the column.
    ///  Defaults to the generated SQL to create a
    /// column of the inferred type.
    /// </summary>
    public string ColumnDefinition = "";

    /// <summary> (Optional) The name of the table that contains the column. 
    /// If absent the column is assumed to be in the primary table.
    /// </summary>
    public string Table = "";

    /// <summary> (Optional) The column length. (Applies only if a
    /// string-valued column is used.)
    /// </summary>
    public int Length = 255;

    /// <summary> (Optional) The precision for a decimal (exact numeric) 
    /// column. (Applies only if a decimal column is used.)
    /// Value must be set by developer if used when generating 
    /// the DDL for the column.
    /// </summary>
    public int Precision = 0;

    /// <summary> (Optional) The scale for a decimal (exact numeric) column. 
    /// (Applies only if a decimal column is used.)
    /// </summary>
    public int Scale = 0;
}
}