/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;


/// <summary> Interface used to interact with the persistence context.
/// <para>
///  An <c>EntityManager</c> instance is associated with 
/// a persistence context. A persistence context is a set of entity 
/// instances in which for any persistent entity identity there is 
/// a unique entity instance. Within the persistence context, the 
/// entity instances and their lifecycle are managed. 
/// The <c>EntityManager</c> API is used 
/// to create and remove persistent entity instances, to find entities 
/// by their primary key, and to query over entities.
/// </para>
/// <para>
///  The set of entities that can be managed by a given 
/// <c>EntityManager</c> instance is defined by a persistence 
/// unit. A persistence unit defines the set of all classes that are 
/// related or grouped by the application, and which must be 
/// colocated in their mapping to a single database.
/// </para>
/// <seealso cref="Query"/>
/// <seealso cref="T:TypedQuery`1"/>
/// <!--seealso cref="CriteriaQuery"/-->
/// <seealso cref="PersistenceContext"/>
/// 
/// 
/// </summary>
public interface EntityManager {

    /// <summary> Make an instance managed and persistent.</summary>
    /// <param name="entity">  entity instance</param>
    /// <exception cref="EntityExistsException"> if the entity already exists.
    /// (If the entity already exists, the <c>EntityExistsException</c> may 
    /// be thrown when the persist operation is invoked, or the
    /// <c>EntityExistsException</c> or another <c>PersistenceException</c> may be 
    /// thrown at flush or commit time.) </exception>
    /// <exception cref="ArgumentException"> if the instance is not an
    ///         entity</exception>
    /// <exception cref="TransactionRequiredException"> if invoked on a
    ///         container-managed entity manager of type 
    ///         <c>PersistenceContextType.TRANSACTION</c> and there is 
    ///         no transaction</exception>
    /// 
    void Persist(object entity);
    
    /// <summary> Merge the state of the given entity into the
    /// current persistence context.</summary>
    /// <param name="entity">  entity instance</param>
    /// <returns> the managed instance that the state was merged to</returns>
    /// <exception cref="ArgumentException"> if instance is not an
    ///         entity or is a removed entity</exception>
    /// <exception cref="TransactionRequiredException"> if invoked on a
    ///         container-managed entity manager of type 
    ///         <c>PersistenceContextType.TRANSACTION</c> and there is 
    ///         no transaction</exception>
    T Merge<T>(T entity) where T : class;

    /// <summary> Remove the entity instance.</summary>  
    /// <param name="entity">  entity instance</param>
    /// <exception cref="ArgumentException"> if the instance is not an</exception>
    ///         entity or is a detached entity
    /// <exception cref="TransactionRequiredException"> if invoked on a
    ///         container-managed entity manager of type 
    ///         <c>PersistenceContextType.TRANSACTION</c> and there is 
    ///         no transaction</exception>
    ///   
    void Remove(object entity);
    
    /// <summary> Find by primary key.
    /// Search for an entity of the specified class and primary key.
    /// If the entity instance is contained in the persistence context,
    /// it is returned from there.</summary>  
    /// <param name="entityClass">  entity class</param>
    /// <param name="primaryKey">  primary key</param>
    /// <returns> the found entity instance or null if the entity does
    ///         not exist</returns>
    /// <exception cref="ArgumentException"> if the first argument does
    ///         not denote an entity type or the second argument is 
    ///         is not a valid type for that entitys primary key or
    ///         is null</exception>
    
    T Find<T> (System.Type entityClass, object primaryKey);
    
    /// <summary> Find by primary key, using the specified properties. 
    /// Search for an entity of the specified class and primary key. 
    /// If the entity instance is contained in the persistence 
    /// context, it is returned from there. 
    /// If a vendor-specific property or hint is not recognized, 
    /// it is silently ignored. </summary>  
    /// <param name="entityClass">  entity class</param>
    /// <param name="primaryKey">   primary key</param>
    /// <param name="properties">  standard and vendor-specific properties 
    ///        and hints</param>
    /// <returns> the found entity instance or null if the entity does
    ///         not exist </returns>
    /// <exception cref="ArgumentException"> if the first argument does 
    ///         not denote an entity type or the second argument is
    ///         is not a valid type for that entitys primary key or 
    ///         is null </exception>
    
    T Find<T> (System.Type entityClass, object primaryKey, 
                      Dictionary<string, object> properties); 
    
    /// <summary> Find by primary key and lock.
    /// Search for an entity of the specified class and primary key
    /// and lock it with respect to the specified lock type.
    /// If the entity instance is contained in the persistence context,
    /// it is returned from there, and the effect of this method is
    /// the same as if the lock method had been called on the entity.
    ///  If the entity is found within the persistence context and the
    /// lock mode type is pessimistic and the entity has a version
    /// attribute, the persistence provider must perform optimistic
    /// version checks when obtaining the database lock.  If these 
    /// checks fail, the <c>OptimisticLockException</c> will be thrown.
    /// If the lock mode type is pessimistic and the entity instance
    /// is found but cannot be locked:
    /// <ul>
    /// <li> the <c>PessimisticLockException</c> will be thrown if the database
    ///    locking failure causes transaction-level rollback
    ///    </li>
    /// <li> the <c>LockTimeoutException</c> will be thrown if the database
    ///    locking failure causes only statement-level rollback
    ///    </li>
    /// </ul></summary>  
    /// <param name="entityClass">  entity class</param>
    /// <param name="primaryKey">  primary key</param>
    /// <param name="lockMode">  lock mode</param>
    /// <returns> the found entity instance or null if the entity does
    ///         not exist</returns>
    /// <exception cref="ArgumentException"> if the first argument does
    ///         not denote an entity type or the second argument is 
    ///         not a valid type for that entity's primary key or 
    ///         is null</exception>
    /// <exception cref="TransactionRequiredException"> if there is no 
    ///         transaction and a lock mode other than NONE is
    ///         specified</exception>
    /// <exception cref="OptimisticLockException"> if the optimistic version 
    ///         check fails</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking 
    ///         fails and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking fails and
    ///         only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if an unsupported lock call 
    ///         is made</exception>
    /// 
    T Find<T> (System.Type entityClass, object primaryKey,
                      LockModeType lockMode);

    /// <summary> Find by primary key and lock, using the specified properties. 
    /// Search for an entity of the specified class and primary key
    /// and lock it with respect to the specified lock type.
    /// If the entity instance is contained in the persistence context,
    /// it is returned from there.  
    ///  If the entity is found
    /// within the persistence context and the lock mode type
    /// is pessimistic and the entity has a version attribute, the
    /// persistence provider must perform optimistic version checks
    /// when obtaining the database lock.  If these checks fail,
    /// the <c>OptimisticLockException</c> will be thrown.
    /// If the lock mode type is pessimistic and the entity instance
    /// is found but cannot be locked:
    /// <ul>
    /// <li> the <c>PessimisticLockException</c> will be thrown if the database
    ///    locking failure causes transaction-level rollback</li>
    /// <li> the <c>LockTimeoutException</c> will be thrown if the database
    ///    locking failure causes only statement-level rollback</li>
    /// </ul>
    /// If a vendor-specific property or hint is not recognized, 
    /// it is silently ignored.  
    /// Portable applications should not rely on the standard timeout
    /// hint. Depending on the database in use and the locking
    /// mechanisms used by the provider, the hint may or may not
    /// be observed.</summary>  
    /// <param name="entityClass">  entity class</param>
    /// <param name="primaryKey">  primary key</param>
    /// <param name="lockMode">  lock mode</param>
    /// <param name="properties">  standard and vendor-specific properties</param>
    ///        and hints
    /// <returns> the found entity instance or null if the entity does
    ///         not exist</returns>
    /// <exception cref="ArgumentException"> if the first argument does
    ///         not denote an entity type or the second argument is 
    ///         not a valid type for that entity's primary key or 
    ///         is null</exception>
    /// <exception cref="TransactionRequiredException"> if there is no 
    ///         transaction and a lock mode other than <c>NONE</c> is 
    ///         specified</exception>
    /// <exception cref="OptimisticLockException"> if the optimistic version 
    ///         check fails</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking 
    ///         fails and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking fails and
    ///         only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if an unsupported lock call 
    ///         is made</exception>
    /// 
    T Find<T> (System.Type entityClass, object primaryKey,
                      LockModeType lockMode, 
                      Dictionary<string, object> properties);

    /// <summary> Get an instance, whose state may be lazily fetched.
    /// If the requested instance does not exist in the database,
    /// the <c>EntityNotFoundException</c> is thrown when the instance 
    /// state is first accessed. (The persistence provider runtime is 
    /// permitted to throw the <c>EntityNotFoundException</c> when 
    /// <c>getReference</c> is called.)
    /// The application should not expect that the instance state will
    /// be available upon detachment, unless it was accessed by the
    /// application while the entity manager was open.</summary>  
    /// <param name="entityClass">  entity class</param>
    /// <param name="primaryKey">  primary key</param>
    /// <returns> the found entity instance</returns>
    /// <exception cref="System.ArgumentException"> if the first argument does
    ///         not denote an entity type or the second argument is
    ///         not a valid type for that entitys primary key or
    ///         is null</exception>
    /// <exception cref="NPersistence.EntityNotFoundException"> if the entity state 
    ///         cannot be accessed</exception>
    
    T GetReference<T> (System.Type entityClass, 
                                  object primaryKey);

    /// <summary> Synchronize the persistence context to the
    /// underlying database.</summary>  
    /// <exception cref="TransactionRequiredException"> if there is
    ///         no transaction</exception>
    /// <exception cref="PersistenceException"> if the flush fails</exception>
    
    void Flush();

    /// <summary> Set the flush mode that applies to all objects contained
    /// in the persistence context. </summary>  
    /// <param name="flushMode">  flush mode</param>
    
    void SetFlushMode(FlushModeType flushMode);

    /// <summary> Get the flush mode that applies to all objects contained
    /// in the persistence context. </summary>  
    /// <returns> flushMode</returns>
    FlushModeType GetFlushMode();

    /// <summary> Lock an entity instance that is contained in the persistence
    /// context with the specified lock mode type.
    /// If a pessimistic lock mode type is specified and the entity
    /// contains a version attribute, the persistence provider must 
    /// also perform optimistic version checks when obtaining the 
    /// database lock.  If these checks fail, the 
    /// <c>OptimisticLockException</c> will be thrown.
    /// If the lock mode type is pessimistic and the entity instance
    /// is found but cannot be locked:
    /// <ul>
    /// <li> the <c>PessimisticLockException</c> will be thrown if the database
    ///    locking failure causes transaction-level rollback</li>
    /// <li> the <c>LockTimeoutException</c> will be thrown if the database
    ///    locking failure causes only statement-level rollback</li>
    /// </ul></summary>  
    /// <param name="entity">  entity instance</param>
    /// <param name="lockMode">  lock mode</param>
    /// <exception cref="ArgumentException"> if the instance is not an
    ///         entity or is a detached entity</exception>
    /// <exception cref="TransactionRequiredException"> if there is no 
    ///         transaction</exception>
    /// <exception cref="EntityNotFoundException"> if the entity does not exist 
    ///         in the database when pessimistic locking is 
    ///         performed</exception>
    /// <exception cref="OptimisticLockException"> if the optimistic version 
    ///         check fails</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking fails 
    ///         and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking fails and
    ///         only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if an unsupported lock call 
    ///         is made</exception>
    void Lock(object entity, LockModeType lockMode);

    /// <summary> Lock an entity instance that is contained in the persistence
    /// context with the specified lock mode type and with specified
    /// properties.
    /// If a pessimistic lock mode type is specified and the entity
    /// contains a version attribute, the persistence provider must 
    /// also perform optimistic version checks when obtaining the 
    /// database lock.  If these checks fail, the 
    /// <c>OptimisticLockException</c> will be thrown.
    /// If the lock mode type is pessimistic and the entity instance
    /// is found but cannot be locked:
    /// <ul>
    /// <li> the <c>PessimisticLockException</c> will be thrown if the database
    ///    locking failure causes transaction-level rollback</li>
    /// <li> the <c>LockTimeoutException</c> will be thrown if the database
    ///    locking failure causes only statement-level rollback</li>
    /// </ul>
    /// If a vendor-specific property or hint is not recognized, 
    /// it is silently ignored.  
    /// Portable applications should not rely on the standard timeout
    /// hint. Depending on the database in use and the locking
    /// mechanisms used by the provider, the hint may or may not
    /// be observed.</summary>  
    /// <param name="entity">  entity instance</param>
    /// <param name="lockMode">  lock mode</param>
    /// <param name="properties">  standard and vendor-specific properties
    ///        and hints</param>
    /// <exception cref="ArgumentException"> if the instance is not an
    ///         entity or is a detached entity</exception>
    /// <exception cref="TransactionRequiredException"> if there is no 
    ///         transaction</exception>
    /// <exception cref="EntityNotFoundException"> if the entity does not exist
    ///         in the database when pessimistic locking is 
    ///         performed</exception>
    /// <exception cref="OptimisticLockException"> if the optimistic version 
    ///         check fails</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking fails
    ///         and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking fails and
    ///         only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if an unsupported lock call 
    ///         is made</exception>
    /// 
    void Lock(object entity, LockModeType lockMode,
                     Dictionary<string, object> properties);

    /// <summary> Refresh the state of the instance from the database, 
    /// overwriting changes made to the entity, if any. </summary>  
    /// <param name="entity">  entity instance</param>
    /// <exception cref="ArgumentException"> if the instance is not
    ///         an entity or the entity is not managed</exception>
    /// <exception cref="TransactionRequiredException"> if invoked on a
    ///         container-managed entity manager of type 
    ///         <c>PersistenceContextType.TRANSACTION</c> and there is 
    ///         no transaction</exception>
    /// <exception cref="EntityNotFoundException"> if the entity no longer
    ///         exists in the database</exception>
    void Refresh(object entity);

    /// <summary> Refresh the state of the instance from the database, using 
    /// the specified properties, and overwriting changes made to
    /// the entity, if any. 
    ///  If a vendor-specific property or hint is not recognized, 
    /// it is silently ignored. </summary>  
    /// <param name="entity">  entity instance</param>
    /// <param name="properties">  standard and vendor-specific properties 
    ///        and hints</param>
    /// <exception cref="ArgumentException"> if the instance is not 
    ///         an entity or the entity is not managed </exception>
    /// <exception cref="TransactionRequiredException"> if invoked on a 
    ///         container-managed entity manager of type
    ///         <c>PersistenceContextType.TRANSACTION</c> and there is
    ///         no transaction </exception>
    /// <exception cref="EntityNotFoundException"> if the entity no longer 
    ///         exists in the database </exception>
    ///    
    void Refresh(object entity,
                            Dictionary<string, object> properties); 

    /// <summary> Refresh the state of the instance from the database, 
    /// overwriting changes made to the entity, if any, and 
    /// lock it with respect to given lock mode type.
    /// If the lock mode type is pessimistic and the entity instance
    /// is found but cannot be locked:
    /// <ul>
    /// <li> the <c>PessimisticLockException</c> will be thrown if the database
    ///    locking failure causes transaction-level rollback</li>
    /// <li> the <c>LockTimeoutException</c> will be thrown if the
    ///    database locking failure causes only statement-level </li>
    ///    rollback.
    /// </ul></summary>  
    /// <param name="entity">  entity instance</param>
    /// <param name="lockMode">  lock mode</param>
    /// <exception cref="ArgumentException"> if the instance is not
    ///         an entity or the entity is not managed</exception>
    /// <exception cref="TransactionRequiredException"> if there is no 
    ///         transaction and if invoked on a container-managed
    ///         <c>EntityManager</c> instance with
    ///         <c>PersistenceContextType.TRANSACTION</c> or with a lock mode 
    ///         other than <c>NONE</c></exception>
    /// <exception cref="EntityNotFoundException"> if the entity no longer exists
    ///         in the database</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking fails
    ///         and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking fails and
    ///         only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if an unsupported lock call
    ///         is made</exception>
    /// 
    void Refresh(object entity, LockModeType lockMode);

    /// <summary> Refresh the state of the instance from the database, 
    /// overwriting changes made to the entity, if any, and 
    /// lock it with respect to given lock mode type and with
    /// specified properties.
    /// If the lock mode type is pessimistic and the entity instance
    /// is found but cannot be locked:
    /// <ul>
    /// <li> the <c>PessimisticLockException</c> will be thrown if the database
    ///    locking failure causes transaction-level rollback</li>
    /// <li> the <c>LockTimeoutException</c> will be thrown if the database
    ///    locking failure causes only statement-level rollback</li>
    /// </ul>
    /// If a vendor-specific property or hint is not recognized, 
    ///    it is silently ignored.  
    /// Portable applications should not rely on the standard timeout
    /// hint. Depending on the database in use and the locking
    /// mechanisms used by the provider, the hint may or may not
    /// be observed.</summary>  
    /// <param name="entity">  entity instance</param>
    /// <param name="lockMode">  lock mode</param>
    /// <param name="properties">  standard and vendor-specific properties</param>
    ///        and hints
    /// <exception cref="ArgumentException"> if the instance is not
    ///         an entity or the entity is not managed</exception>
    /// <exception cref="TransactionRequiredException"> if there is no 
    ///         transaction and if invoked on a container-managed
    ///         <c>EntityManager</c> instance with
    ///         <c>PersistenceContextType.TRANSACTION</c> or with a lock mode 
    ///         other than <c>NONE</c></exception>
    /// <exception cref="EntityNotFoundException"> if the entity no longer exists
    ///         in the database</exception>
    /// <exception cref="PessimisticLockException"> if pessimistic locking fails
    ///         and the transaction is rolled back</exception>
    /// <exception cref="LockTimeoutException"> if pessimistic locking fails and
    ///         only the statement is rolled back</exception>
    /// <exception cref="PersistenceException"> if an unsupported lock call
    ///         is made</exception>
    /// 
    void Refresh(object entity, LockModeType lockMode,
                        Dictionary<string, object> properties);
    
    /// <summary> Clear the persistence context, causing all managed
    /// entities to become detached. Changes made to entities that
    /// have not been flushed to the database will not be
    /// persisted.
    /// </summary>
    void Clear();

    /// <summary> Remove the given entity from the persistence context, causing
    /// a managed entity to become detached.  Unflushed changes made 
    /// to the entity if any (including removal of the entity), 
    /// will not be synchronized to the database.  Entities which 
    /// previously referenced the detached entity will continue to
    /// reference it.</summary>  
    /// <param name="entity">  entity instance</param>
    /// <exception cref="ArgumentException"> if the instance is not an 
    ///         entity </exception>
    /// 
    void Detach(object entity); 

    /// <summary> Check if the instance is a managed entity instance belonging
    /// to the current persistence context.</summary>  
    /// <param name="entity">  entity instance</param>
    /// <returns> boolean indicating if entity is in persistence context</returns>
    /// <exception cref="ArgumentException"> if not an entity</exception>
    bool Contains(object entity);

    /// <summary> Get the current lock mode for the entity instance.</summary>  
    /// <param name="entity">  entity instance</param>
    /// <returns> lock mode</returns>
    /// <exception cref="TransactionRequiredException"> if there is no 
    ///         transaction</exception>
    /// <exception cref="ArgumentException"> if the instance is not a
    ///         managed entity and a transaction is active</exception>
    /// 
    LockModeType GetLockMode(object entity);

    /// <summary> 
    /// Set an entity manager property or hint. 
    /// If a vendor-specific property or hint is not recognized, it is
    /// silently ignored. </summary>  
    /// <param name="propertyName"> name of property or hint</param>
    /// <param name="value">  value for property or hint</param>
    /// <exception cref="ArgumentException"> if the second argument is 
    ///         not valid for the implementation </exception>
    /// 
    void SetProperty(string propertyName, object value);

    /// <summary> Get the properties and hints and associated values that are in effect 
    /// for the entity manager. Changing the contents of the map does 
    /// not change the configuration in effect.</summary>  
    /// <returns> map of properties and hints in effect for entity manager</returns>
    /// 
    Dictionary<string, object> GetProperties();

    /// <summary> Create an instance of <c>Query</c> for executing a
    /// Java Persistence query language statement.</summary>  
    /// <param name="qlString"> a Java Persistence query string</param>
    /// <returns> the new query instance</returns>
    /// <exception cref="ArgumentException"> if the query string is
    ///         found to be invalid</exception>
    Query CreateQuery(string qlString);

    /// <summary> Create an instance of <c>TypedQuery</c> for executing a
    /// criteria query.</summary>  
    /// <param name="criteriaQuery">  a criteria query object</param>
    /// <returns> the new query instance</returns>
    /// <exception cref="ArgumentException"> if the criteria query is
    ///         found to be invalid</exception>
    /// 
//    TypedQuery<T> CreateQuery<T> (CriteriaQuery<T> criteriaQuery); 

    /// <summary> Create an instance of <c>TypedQuery</c> for executing a
    /// Java Persistence query language statement.
    /// The select list of the query must contain only a single
    /// item, which must be assignable to the type specified by
    /// the <c>resultClass</c> argument.</summary>  
    /// <param name="qlString"> a Java Persistence query string</param>
    /// <param name="resultClass"> the type of the query result</param>
    /// <returns> the new query instance</returns>
    /// <exception cref="ArgumentException"> if the query string is found
    ///         to be invalid or if the query result is found to
    ///         not be assignable to the specified type</exception>
    /// 
    TypedQuery<T> CreateQuery<T> (string qlString, System.Type resultClass);

    /// <summary> Create an instance of <c>Query</c> for executing a named query
    /// (in the Java Persistence query language or in native SQL).</summary>  
    /// <param name="name"> the name of a query defined in metadata</param>
    /// <returns> the new query instance</returns>
    /// <exception cref="ArgumentException"> if a query has not been
    ///         defined with the given name or if the query string is
    ///         found to be invalid</exception>
    Query CreateNamedQuery(string name);

    /// <summary> Create an instance of <c>TypedQuery</c> for executing a
    /// Java Persistence query language named query.
    /// The select list of the query must contain only a single
    /// item, which must be assignable to the type specified by
    /// the <c>resultClass</c> argument.</summary>  
    /// <param name="name"> the name of a query defined in metadata</param>
    /// <param name="resultClass"> the type of the query result</param>
    /// <returns> the new query instance</returns>
    /// <exception cref="ArgumentException"> if a query has not been
    ///         defined with the given name or if the query string is
    ///         found to be invalid or if the query result is found to
    ///         not be assignable to the specified type</exception>
    /// 
    TypedQuery<T> CreateNamedQuery<T> (string name, System.Type resultClass);

    /// <summary> Create an instance of <c>Query</c> for executing
    /// a native SQL statement, e.g., for update or delete.</summary>  
    /// <param name="sqlString"> a native SQL query string</param>
    /// <returns> the new query instance</returns>
    Query CreateNativeQuery(string sqlString);

    /// <summary> Create an instance of <c>Query</c> for executing
    /// a native SQL query.</summary>  
    /// <param name="sqlString"> a native SQL query string</param>
    /// <param name="resultClass"> the class of the resulting instance(s)</param>
    /// <returns> the new query instance</returns>
    
    Query CreateNativeQuery(string sqlString, System.Type resultClass);

    /// <summary> Create an instance of <c>Query</c> for executing
    /// a native SQL query.</summary>  
    /// <param name="sqlString"> a native SQL query string</param>
    /// <param name="resultSetMapping"> the name of the result set mapping</param>
    /// <returns> the new query instance</returns>
    Query CreateNativeQuery(string sqlString, string resultSetMapping);

    /// <summary> Indicate to the entity manager that a JTA transaction is
    /// active. This method should be called on a JTA application 
    /// managed entity manager that was created outside the scope
    /// of the active transaction to associate it with the current
    /// JTA transaction.</summary>  
    /// <exception cref="TransactionRequiredException"> if there is
    ///         no transaction</exception>
    void JoinTransaction();

    /// <summary> Return an object of the specified type to allow access to the
    /// provider-specific API.   If the provider's <c>EntityManager</c>
    /// implementation does not support the specified class, the
    /// <c>PersistenceException</c> is thrown.</summary>  
    /// <param name="cls">  the class of the object to be returned.  This is</param>
    /// normally either the underlying <c>EntityManager</c> implementation
    /// class or an interface that it implements.
    /// <returns> an instance of the specified class</returns>
    /// <exception cref="PersistenceException"> if the provider does not 
    ///         support the call </exception>
    /// 
    T Unwrap<T> (System.Type cls); 

    /// <summary> Return the underlying provider object for the <c>EntityManager</c>,
    /// if available. The result of this method is implementation
    /// specific. The <c>unwrap</c> method is to be preferred for new 
    /// applications.</summary>  
    /// <returns> underlying provider object for EntityManager</returns>
    object GetDelegate();

    /// <summary> Close an application-managed entity manager. 
    /// After the close method has been invoked, all methods
    /// on the <c>EntityManager</c> instance and any 
    /// <c>Query</c> and <c>TypedQuery</c>
    /// objects obtained from it will throw the <c>InvalidOperationException</c>
    /// except for <c>getProperties</c>, 
    /// <c>getTransaction</c>, and <c>isOpen</c> (which will return false).
    /// If this method is called when the entity manager is
    /// associated with an active transaction, the persistence
    /// context remains managed until the transaction completes. </summary>  
    /// <exception cref="InvalidOperationException"> if the entity manager
    ///         is container-managed</exception>
    void Close();

    /// <summary> Determine whether the entity manager is open. </summary>  
    /// <returns> true until the entity manager has been closed</returns>
    
    bool IsOpen();

    /// <summary> Return the resource-level <c>EntityTransaction</c> object. 
    /// The <c>EntityTransaction</c> instance may be used serially to 
    /// begin and commit multiple transactions.</summary>  
    /// <returns> EntityTransaction instance</returns>
    /// <exception cref="InvalidOperationException"> if invoked on a JTA
    ///         entity manager</exception>
    EntityTransaction GetTransaction();

    /// <summary> Return the entity manager factory for the entity manager.</summary>  
    /// <returns> EntityManagerFactory instance</returns>
    /// <exception cref="InvalidOperationException"> if the entity manager has 
    ///         been closed</exception>
    /// 
    EntityManagerFactory GetEntityManagerFactory();

    /// <summary> Return an instance of <c>CriteriaBuilder</c> for the creation of
    /// <c>CriteriaQuery</c> objects.</summary>  
    /// <returns> CriteriaBuilder instance</returns>
    /// <exception cref="InvalidOperationException"> if the entity manager has
    ///         been closed</exception>
    /// 
//    CriteriaBuilder GetCriteriaBuilder();

    /// <summary> Return an instance of <c>Metamodel</c> interface for access to the
    /// metamodel of the persistence unit.</summary>  
    /// <returns> Metamodel instance</returns>
    /// <exception cref="InvalidOperationException"> if the entity manager has
    ///         been closed</exception>
    /// 
//    Metamodel GetMetamodel();
}
}