/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;



/// <summary> Specifies a column for joining an entity association or element
/// collection.  
    /// <para>
    /// If the <c>JoinColumn</c> annotation itself is
/// defaulted, a single join column is assumed and the default values
/// apply.
/// </para>
/// <example><code>
///   Example:
/// 
///   &#91;ManyToOne&#93;
///   &#91;JoinColumn(name="ADDR_ID")&#93;
///   public Address getAddress() { return address; }
/// 
/// 
///   Example: unidirectional one-to-many association using a foreign key mapping
/// 
///   // In Customer class
///   &#91;OneToMany&#93;
///   &#91;JoinColumn(name="CUST_ID")&#93; // join column is in table for Order
///   public Set&#060;Order&#062; getOrders() {return orders;}
/// </code></example>
/// 
/// <seealso cref="ManyToOne"/>
/// <seealso cref="OneToMany"/>
/// <seealso cref="OneToOne"/>
/// <seealso cref="JoinTable"/>
/// <seealso cref="CollectionTable"/>
/// </summary>
[System.AttributeUsage(System.AttributeTargets.Field | System.AttributeTargets.Property | System.AttributeTargets.Class, AllowMultiple=true)]

public sealed class JoinColumn : System.Attribute {

    /// <summary>
   /// (Optional)
   /// Points to the JoinTable, CollectionTable or AssociationOverride attribute which this JoinColumn belongs to.
   /// If this JoinColumn is not part of a JoinTable, CollectionTable or AssociationOverride, this field should stay empty.
   /// 
   /// If this field is populated and references a JoinTable, this JoinColumn represents a foreign key column
   /// of the join table which reference the
   /// primary table of the entity owning the
   /// association. (I.e. the owning side of
   /// the association).
   /// 
   /// If this field is populated and references a collection table,
   ///  this JoinColumn references the primary table of the entity. The default
   ///  only applies if a single join column is used.  The default is
   ///  the same as for <c>JoinColumn</c> (i.e., the
   ///  concatenation of the following: the name of the entity; "_";
   ///  the name of the referenced primary key column.) However, if
   ///  there is more than one join column, a <c>JoinColumn</c>
   ///  annotation must be specified for each join column . In this case, both the
   ///  <c>name</c> and the <c>referencedColumnName</c>
   ///  elements must be specified in each such
   ///  <c>JoinColumn</c> annotation. 
   ///  
   /// If this field is populated and references an AssociationOverride, this JoinColumn represents a join column
   /// being mapped to a persistent attribute.
   /// The <c>joinColumn</c> elements must be specified if a
   /// foreign key mapping is used in the overriding of the mapping of
   /// the relationship.  The <c>joinColumn</c> element must
   /// not be specified if a join table is used in the overriding of
   /// the mapping of the relationship.
   /// </summary>
    public String AttributeRef = null;

    /// <summary> (Optional) Only relevant when this JoinColumn is part of a JoinTable.
    /// This field indicates that this JoinColumn represents the foreign key
    /// of the join table which references the
    /// primary table of the entity that does
    /// not own the association. (I.e. the
    /// inverse side of the association).
    /// </summary>
    public Boolean Inverse = false;

    /// <summary> 
    /// (Optional) The name of the foreign key column.
    /// The table in which it is found depends upon the
    /// context. 
    /// <ul>
    /// <li>If the join is for a OneToOne or ManyToOne
    ///  mapping using a foreign key mapping strategy, 
    /// the foreign key column is in the table of the
    /// source entity or embeddable. </li>
    /// <li> If the join is for a unidirectional OneToMany mapping
    /// using a foreign key mapping strategy, the foreign key is in the
    /// table of the target entity.  </li>
    /// <li> If the join is for a ManyToMany mapping or for a OneToOne
    /// or bidirectional ManyToOne/OneToMany mapping using a join
    /// table, the foreign key is in a join table.  </li>
    /// <li> If the join is for an element collection, the foreign 
    /// key is in a collection table.</li>
    /// </ul>
    /// 
    ///  Default (only applies if a single join column is used):
    /// The concatenation of the following: the name of the 
    /// referencing relationship property or field of the referencing 
    /// entity or embeddable class; "_"; the name of the referenced 
    /// primary key column. 
    /// If there is no such referencing relationship property or 
    /// field in the entity, or if the join is for an element collection,
    /// the join column name is formed as the 
    /// concatenation of the following: the name of the entity; "_"; 
    /// the name of the referenced primary key column.
    /// </summary>
    public string Name = "";

    /// <summary> (Optional) The name of the column referenced by this foreign
    /// key column. 
    /// <ul>
    /// <li> When used with entity relationship mappings other
    /// than the cases described here, the referenced column is in the
    /// table of the target entity. </li>
    /// <li> When used with a unidirectional OneToMany foreign key
    /// mapping, the referenced column is in the table of the source
    /// entity.  </li>
    /// <li> When used inside a <c>JoinTable</c> annotation,
    /// the referenced key column is in the entity table of the owning
    /// entity, or inverse entity if the join is part of the inverse
    /// join definition.  </li>
    /// <li> When used in a <c>CollectionTable</c> mapping, the
    /// referenced column is in the table of the entity containing the
    /// collection.</li>
    /// </ul>
    /// 
    ///  Default (only applies if single join column is being 
    /// used): The same name as the primary key column of the 
    /// referenced table.
    /// </summary>
    public string ReferencedColumnName = "";

    /// <summary> (Optional) Whether the property is a unique key.  This is a
    /// shortcut for the <c>UniqueConstraint</c> annotation at
    /// the table level and is useful for when the unique key
    /// constraint is only a single field. It is not necessary to
    /// explicitly specify this for a join column that corresponds to a
    /// primary key that is part of a foreign key.
    /// </summary>
    public bool Unique = false;

    /// <summary> (Optional) Whether the foreign key column is nullable.
    /// </summary>
    public bool Nullable = true;

    /// <summary> (Optional) Whether the column is included in
    /// SQL INSERT statements generated by the persistence
    /// provider.
    /// </summary>
    public bool Insertable = true;

    /// <summary> (Optional) Whether the column is included in
    /// SQL UPDATE statements generated by the persistence
    /// provider.
    /// </summary>
    public bool Updatable = true;

    /// <summary> (Optional) The SQL fragment that is used when
    /// generating the DDL for the column.
    ///  Defaults to the generated SQL for the column.
    /// </summary>
    public string ColumnDefinition = "";

    /// <summary> (Optional) The name of the table that contains
    /// the column. If a table is not specified, the column
    /// is assumed to be in the primary table of the
    /// applicable entity.
    /// 
    ///  Default: 
    /// <ul>
    /// <li> If the join is for a OneToOne or ManyToOne mapping
    /// using a foreign key mapping strategy, the name of the table of
    /// the source entity or embeddable. </li>
    /// <li> If the join is for a unidirectional OneToMany mapping 
    /// using a foreign key mapping strategy, the name of the table of
    /// the target entity. </li>
    /// <li> If the join is for a ManyToMany mapping or
    /// for a OneToOne or bidirectional ManyToOne/OneToMany mapping
    /// using a join table, the name of the join table. </li>
    /// <li> If the join is for an element collection, the name of the collection table.</li>
    /// </ul>
    /// </summary>
    public string Table = "";
}
}