/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

    /// <summary> 
    /// Lock modes can be specified by means of passing a <c>LockModeType</c>
    /// argument to one of the <see cref="NPersistence.EntityManager"/> methods that take locks
    /// (<c>lock</c>, <c>find</c>, or <c>refresh</c>) or
    /// to the <see cref="Query.SetLockMode"/> or
    /// <see cref="M:TypedQuery`1.SetLockMode"/> method.
    /// <para>
    ///  Lock modes can be used to specify either optimistic or pessimistic locks.
    /// </para>
    /// <para>
    ///  Optimistic locks are specified using 
    /// <see cref="LockModeType.OPTIMISTIC"/> and 
    /// <see cref="LockModeType.OPTIMISTIC_FORCE_INCREMENT"/>.
    /// </para>
    /// <para>
    /// The lock mode type
    /// values <see cref="LockModeType.READ"/> and 
    /// <see cref="LockModeType.WRITE"/> are
    /// synonyms of <c>OPTIMISTIC</c> and
    /// <c>OPTIMISTIC_FORCE_INCREMENT</c> respectively. 
    /// </para><para>
    /// The latter
    /// are to be preferred for new applications.
    /// </para>
    /// <para>
    ///  The semantics of requesting locks of type
    /// <c>LockModeType.OPTIMISTIC</c> and
    /// <c>LockModeType.OPTIMISTIC_FORCE_INCREMENT</c> are the
    /// following.
    /// </para>
    /// <para>
    ///  If transaction T1 calls for a lock of type 
    /// <c>LockModeType.OPTIMISTIC</c> on a versioned object, 
    /// the entity manager must ensure that neither of the following 
    /// phenomena can occur:
    /// </para>
    /// <ul>
    ///   <li> P1 (Dirty read): Transaction T1 modifies a row. 
    /// Another transaction T2 then reads that row and obtains 
    /// the modified value, before T1 has committed or rolled back. 
    /// Transaction T2 eventually commits successfully; it does not 
    /// matter whether T1 commits or rolls back and whether it does 
    /// so before or after T2 commits.
    ///   <li>
    ///   </li> P2 (Non-repeatable read): Transaction T1 reads a row. 
    /// Another transaction T2 then modifies or deletes that row, 
    /// before T1 has committed. Both transactions eventually commit 
    /// successfully.
    ///   </li>
    /// </ul>
    /// <para>
    ///  Lock modes must always prevent the phenomena P1 and P2.
    /// </para>
    /// <para>
    ///  In addition, calling a lock of type 
    /// <c>LockModeType.OPTIMISTIC_FORCE_INCREMENT</c> on a versioned object,
    /// will also force an update (increment) to the entity's version
    /// column.
    /// </para>
    /// <para>
    ///  The persistence implementation is not required to support
    /// the use of optimistic lock modes on non-versioned objects. When it
    /// cannot support a such lock call, it must throw the 
    /// <see cref="PersistenceException"/>.
    /// </para>
    /// <para>
    /// The lock modes <see cref="F:LockModeType.PESSIMISTIC_READ"/>, <see cref="F:LockModeType.PESSIMISTIC_WRITE"/>, and
    /// <see cref="LockModeType.PESSIMISTIC_FORCE_INCREMENT"/> are used to immediately
    /// obtain long-term database locks.
    /// </para>
    /// <para>
    ///  The semantics of requesting locks of type
    /// <c>LockModeType.PESSIMISTIC_READ</c>, <c>LockModeType.PESSIMISTIC_WRITE</c>, and
    /// <c>LockModeType.PESSIMISTIC_FORCE_INCREMENT</c> are the following.  
    /// </para>
    /// <para>
    ///  If transaction T1 calls for a lock of type
    /// <c>LockModeType.PESSIMISTIC_READ</c> or
    /// <c>LockModeType.PESSIMISTIC_WRITE</c> on an object, the entity
    /// manager must ensure that neither of the following phenomena can
    /// occur: 
    /// </para>
    /// <ul> 
    /// <li> P1 (Dirty read): Transaction T1 modifies a
    /// row. Another transaction T2 then reads that row and obtains the
    /// modified value, before T1 has committed or rolled back.</li>
/// 
/// <li> P2 (Non-repeatable read): Transaction T1 reads a row. Another
/// transaction T2 then modifies or deletes that row, before T1 has
    /// committed or rolled back.</li>
/// </ul>
    /// <para>
///  A lock with <c>LockModeType.PESSIMISTIC_WRITE</c> can be obtained on
/// an entity instance to force serialization among transactions
/// attempting to update the entity data. A lock with
/// <c>LockModeType.PESSIMISTIC_READ</c> can be used to query data using
/// repeatable-read semantics without the need to reread the data at
/// the end of the transaction to obtain a lock, and without blocking
/// other transactions reading the data. A lock with
/// <c>LockModeType.PESSIMISTIC_WRITE</c> can be used when querying data and
/// there is a high likelihood of deadlock or update failure among
/// concurrent updating transactions.
    /// </para>
    /// <para>
///  The persistence implementation must support use of locks of type
/// <c>LockModeType.PESSIMISTIC_READ</c> 
/// <c>LockModeType.PESSIMISTIC_WRITE</c> on a non-versioned entity as well as
/// on a versioned entity.
    /// </para>
    /// <para>
///  When the lock cannot be obtained, and the database locking
/// failure results in transaction-level rollback, the provider must
/// throw the <see cref="PessimisticLockException"/> and ensure that the JTA
/// transaction or <c>EntityTransaction</c> has been marked for rollback.
    /// </para>
    /// <para>
///  When the lock cannot be obtained, and the database locking
/// failure results in only statement-level rollback, the provider must
    /// throw the <see cref="LockTimeoutException"/> (and must not mark the transaction
/// for rollback).
/// </para>
/// </summary>
public enum LockModeType
{
    /// <summary>  Synonymous with <c>OPTIMISTIC</c>.
    ///  <c>OPTIMISTIC</c> is to be preferred for new
    ///  applications.
    /// 
    /// </summary>
    READ,

    /// <summary>  Synonymous with <c>OPTIMISTIC_FORCE_INCREMENT</c>.
    ///  <c>OPTIMISTIC_FORCE_IMCREMENT</c> is to be preferred for new
    ///  applications.
    /// 
    /// </summary>
    WRITE,

    /// <summary> Optimistic lock.
    /// 
    /// 
    /// </summary>
    OPTIMISTIC,

    /// <summary> Optimistic lock, with version update.
    /// 
    /// 
    /// </summary>
    OPTIMISTIC_FORCE_INCREMENT,

    /// <summary> Not supported for NPA-Nhibernate
    /// 
    /// Pessimistic read lock.
    /// 
    /// 
    /// </summary>
    //PESSIMISTIC_READ,

    /// <summary> Pessimistic write lock.
    /// 
    /// 
    /// </summary>
    PESSIMISTIC_WRITE,

    /// <summary> Pessimistic write lock, with version update.
    /// 
    /// 
    /// </summary>
    PESSIMISTIC_FORCE_INCREMENT,

    /// <summary> No lock.
    /// 
    /// 
    /// </summary>
    NONE
}
}