/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
///     dclarke - EclipseLink implementation and default resolver
/// 
/*****************************************************************************/
namespace NPersistence.spi
{
    using System.Collections;
    using System.Collections.Generic;
    using System.Threading;
    using System.Reflection;
    using System.IO;
    using log4net;
    using log4net.Config;
    using System;


    /// <summary> Holds the global <see cref="NPersistence.spi.PersistenceProviderResolver"/>
    /// instance. If no <c>PersistenceProviderResolver</c> is set by the
    /// environment, the default <c>PersistenceProviderResolver</c> is used.
    /// <para>
    /// Implementations must be thread-safe.
    /// </para>
    /// </summary>
    public class PersistenceProviderResolverHolder
    {

        private static PersistenceProviderResolver m_singleton = new DefaultPersistenceProviderResolver();

        private static IList<PersistenceProvider> m_loadedProviders = null;

        /// <summary>
        /// Returns the current persistence provider resolver.
        /// </summary>
        /// <returns> the current persistence provider resolver
        /// </returns>
        public static PersistenceProviderResolver GetPersistenceProviderResolver()
        {
            return m_singleton;
        }

        /// <summary>
        /// Defines the persistence provider resolver used.
        /// </summary>
        /// <param name="resolver">persistence provider resolver to be used</param> 
        public static void SetPersistenceProviderResolver(PersistenceProviderResolver resolver)
        {
            if (resolver == null)
            {
                m_singleton = new DefaultPersistenceProviderResolver();
            }
            else
            {
                m_singleton = resolver;
            }
        }

        /// <summary>
        /// Default provider resolver class to use when none is explicitly set.
        /// Uses the META-INF/services approach as described in the Java Persistence
        /// specification. A getResources() call is made on the current context
        /// classloader to find the service provider files on the classpath. Any
        /// service files found are then read to obtain the classes that implement
        /// the persistence provider interface.
        /// </summary>
        private class DefaultPersistenceProviderResolver : PersistenceProviderResolver
        {
            public IList<PersistenceProvider> GetPersistenceProviders()
            {
                if (m_loadedProviders != null)
                {
                    return m_loadedProviders;
                }
                m_loadedProviders = new List<PersistenceProvider>();
                IList<Assembly> assemblies = new List<Assembly>();
                Assembly[] loadedAssemblies = AppDomain.CurrentDomain.GetAssemblies();
                
                IList<String> loadedPaths = new List<String>();
                foreach (Assembly assembly in loadedAssemblies)
                {
                    try
                    {
                        loadedPaths.Add(assembly.Location.ToLower());
                        assemblies.Add(assembly);
                    }
                    catch (ReflectionTypeLoadException e)
                    {
                        if (logger.IsErrorEnabled)
                        {
                            logger.Error(e.Message, e);
                            foreach (Exception ex in e.LoaderExceptions)
                            {
                                logger.Error(ex.Message, ex);
                            }
                        }
                    }   
                    catch (Exception e)
                    {
                        if (logger.IsErrorEnabled)
                        {
                            logger.Error(e.Message, e);
                        }
                    }
                }
                String[] referencedPaths = Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory, "*.dll");

                ArrayList toLoad = new ArrayList();
                foreach (String path in referencedPaths)
                {
                    if (!loadedPaths.Contains(path.ToLower()))
                    {
                        Assembly loaded = AppDomain.CurrentDomain.Load(AssemblyName.GetAssemblyName(path));
                        assemblies.Add(loaded);
                    }
                }

                int providersCount = 0;
                
                Type persistenceProviderInterface = typeof(PersistenceProvider);
                
                foreach (Assembly assembly in assemblies)
                {
                    try
                    {
                        foreach (Type type in assembly.GetTypes())
                        {
                            if (persistenceProviderInterface.IsAssignableFrom(type) && !type.IsInterface)
                            {
                                PersistenceProvider provider = (PersistenceProvider)type.GetConstructor(System.Type.EmptyTypes).Invoke((new object[0]));
                                m_loadedProviders.Add(provider);
                                providersCount++;
                            }
                        }
                    }
                    catch (ReflectionTypeLoadException e)
                    {
                        if (logger.IsErrorEnabled)
                        {
                            logger.Error(e.Message, e);
                            foreach (Exception ex in e.LoaderExceptions)
                            {
                                logger.Error(ex.Message, ex);
                            } 
                        }
                    }                    
                }
                
                return m_loadedProviders;
            }

            private const string LOGGER_SUBSYSTEM = "NPersistence.spi";

            private ILog logger = LogManager.GetLogger(LOGGER_SUBSYSTEM);

            private /*static*/ const string SERVICE_PROVIDER_FILE = "META-INF/services/NPersistence.spi.PersistenceProvider";

            /// <summary>
            /// Locate all JPA provider services files and collect all of the
            /// provider names available.
            /// </summary>
            private ICollection<string> GetProviders(System.Reflection.Assembly loader)
            {
                ICollection<string> providerNames = new List<string>();
                try
                {
                    Type persistenceProviderInterface = typeof(PersistenceProvider);
                    Assembly[] assemblies = AppDomain.CurrentDomain. GetAssemblies();
                    foreach(Assembly assembly in assemblies)
                    {
                        foreach (Type type in assembly.GetTypes())
                        {
                            if (persistenceProviderInterface.IsAssignableFrom(type) && !type.IsInterface)
                            {
                                providerNames.Add(type.FullName);
                            }
                        }
                    }

                    
                    //System.IO.Stream stream = loader.GetManifestResourceStream("NPersistence.spi.PersistenceProvider");

                    //StreamReader inp = new StreamReader(stream);

                    //string line;
                    //while ((line = inp.ReadLine()) != null)
                    //{
                    //    providerNames.Add(line);
                    //}
                    //inp.Close();
                }
                catch (System.IO.IOException ioe)
                {
                    throw new PersistenceException("IOException caught: " + loader + ".GetManifestResourceStream(" + SERVICE_PROVIDER_FILE + ")", ioe);
                }
                return providerNames;
            }

            //        private static readonly System.Text.RegularExpressions.Regex nonCommentPattern = Pattern.Compile("^([^#]+)");

            /// <summary>
            /// For each services file look for uncommented provider names on each
            /// line.
            /// </summary>
            /*        private void AddProviderNames(URL url, ICollection<ProviderName> providerNames) {
                        InputStream _in = null;
                        try {
                            _in = url.OpenStream();
                            BufferedReader reader = new BufferedReader(new InputStreamReader(_in));

                            string line;
                            while ((line = reader.ReadLine()) != null) {
                                line = line.Trim();
                                Matcher ___m/*m*/
            /*= nonCommentPattern.Matcher(line);
if (___m/*m*/
            /*(.Find()) {
  providerNames.Add(new ProviderName(___m/*m*/
            /*.Group().Trim(), url));
}
}
} catch (System.IO.IOException ioe) {
throw new PersistenceException("IOException caught reading: " + url, ioe);
} finally {
if (_in != null) {
try {
_in.Close();
} catch (System.IO.IOException ___e/*e*/
            /*) {
}
}
}
}*/

            /// <summary>
            /// Clear all cached providers
            /// </summary>
            public void ClearCachedProviders()
            {
                
            }

        }
    }
}