/*****************************************************************************
/// Copyright (c) 2008, 2009 Sun Microsystems. All rights reserved. 
/// 
/// This program and the accompanying materials are made available under the 
/// terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0 
/// which accompanies this distribution. 
/// The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
/// and the Eclipse Distribution License is available at 
/// http://www.eclipse.org/org/documents/edl-v10.php.
/// 
/// Contributors:
///     Linda DeMichiel - Java Persistence 2.0 - Version 2.0 (October 1, 2009)
///     Specification available from http://jcp.org/en/jsr/detail?id=317
/// 
/*****************************************************************************/
namespace NPersistence.spi { using System; using System.Reflection; using System.Collections; using System.Collections.Generic;

/// <summary> Interface implemented by the container and used by the
    /// persistence provider when creating an <see cref="NPersistence.EntityManagerFactory"/>.
/// 
/// </summary>
public interface PersistenceUnitInfo {
	
    /// <summary> Returns the name of the persistence unit. Corresponds to the
    /// <c>name</c> attribute in the <c>persistence.xml</c> file.
    /// </summary>
    /// <returns>  the name of the persistence unit 
    /// </returns>
    string GetPersistenceUnitName();

    /// <summary> Returns the fully qualified name of the persistence provider
    /// implementation class. Corresponds to the <c>provider</c> element in
    /// the <c>persistence.xml</c> file.</summary>
    /// <returns>  the fully qualified name of the persistence provider 
    /// implementation class</returns>
    /// 
    string GetPersistenceProviderClassName();

    /// <summary> Returns the transaction type of the entity managers created by
    /// the <c>EntityManagerFactory</c>. The transaction type corresponds to
    /// the <c>transaction-type</c> attribute in the <c>persistence.xml</c> file.</summary>
    /// <returns>  transaction type of the entity managers created
    /// by the EntityManagerFactory</returns>
    PersistenceUnitTransactionType GetTransactionType();

    /// <summary> Returns the JTA-enabled data source to be used by the
    /// persistence provider. The data source corresponds to the
    /// <c>jta-data-source</c> element in the <c>persistence.xml</c> file or is
    /// provided at deployment or by the container.</summary>
    /// <returns> the JTA-enabled data source to be used by the 
    /// persistence provider</returns>
    /// 
//    DataSource GetJtaDataSource();

    /// <summary> Returns the non-JTA-enabled data source to be used by the
    /// persistence provider for accessing data outside a JTA
    /// transaction. The data source corresponds to the named
    /// <c>non-jta-data-source</c> element in the <c>persistence.xml</c> file or
    /// provided at deployment or by the container.</summary>
    /// <returns> the non-JTA-enabled data source to be used by the 
    /// persistence provider for accessing data outside a JTA 
    /// transaction</returns>
    /// 
//    DataSource GetNonJtaDataSource();

    /// <summary> Returns the list of the names of the mapping files that the
    /// persistence provider must load to determine the mappings for
    /// the entity classes. The mapping files must be in the standard
    /// XML mapping format, be uniquely named and be resource-loadable
    /// from the application classpath.  Each mapping file name
    /// corresponds to a <c>mapping-file</c> element in the
    /// <c>persistence.xml</c> file.</summary>
    /// <returns> the list of mapping file names that the persistence
    /// provider must load to determine the mappings for the entity
    /// classes </returns>
    /// 
    IList<string> GetMappingFileNames();

    /// <summary> Returns a list of URLs for the jar files or exploded jar
    /// file directories that the persistence provider must examine
    /// for managed classes of the persistence unit. Each URL
    /// corresponds to a <c>jar-file</c> element in the
    /// <c>persistence.xml</c> file. A URL will either be a 
    /// file: URL referring to a jar file or referring to a directory
    /// that contains an exploded jar file, or some other URL from
    /// which an InputStream in jar format can be obtained.
    /// <returns> a list of URL objects referring to jar files or
    /// directories 
    /// 
//    IList<URL> GetJarFileUrls();

    /// <summary> Returns the URL for the jar file or directory that is the
    /// root of the persistence unit. (If the persistence unit is
    /// rooted in the WEB-INF/classes directory, this will be the 
    /// URL of that directory.)
    /// The URL will either be a file: URL referring to a jar file 
    /// or referring to a directory that contains an exploded jar
    /// file, or some other URL from which an InputStream in jar
    /// format can be obtained.</summary>
    /// <returns> a URL referring to a jar file or directory</returns>
    /// 
//    URL GetPersistenceUnitRootUrl();

    /// <summary> Returns the list of the names of the classes that the
    /// persistence provider must add to its set of managed
    /// classes. Each name corresponds to a named <c>class</c> element in the
    /// <c>persistence.xml</c> file.</summary>
    /// <returns> the list of the names of the classes that the 
    /// persistence provider must add to its set of managed 
    /// classes 
    /// </returns>
    IList<string> GetManagedClassNames();

    /// <summary> Returns whether classes in the root of the persistence unit
    /// that have not been explicitly listed are to be included in the
    /// set of managed classes. This value corresponds to the
    /// <c>exclude-unlisted-classes</c> element in the <c>persistence.xml</c> file.</summary>
    /// <returns> whether classes in the root of the persistence
    /// unit that have not been explicitly listed are to be
    /// included in the set of managed classes</returns>
    /// 
    bool ExcludeUnlistedClasses();

    /// <summary> Returns the specification of how the provider must use
    /// a second-level cache for the persistence unit.
    /// The result of this method corresponds to the <c>shared-cache-mode</c>
    /// element in the <c>persistence.xml</c> file.</summary>
    /// <returns> the second-level cache mode that must be used by the
    /// provider for the persistence unit</returns>
    /// 
    /// 
    SharedCacheMode GetSharedCacheMode();

    /// <summary> Returns the validation mode to be used by the persistence
    /// provider for the persistence unit.  The validation mode
    /// corresponds to the <c>validation-mode</c> element in the
    /// <c>persistence.xml</c> file.</summary>
    /// <returns> the validation mode to be used by the 
    /// persistence provider for the persistence unit</returns>
    /// 
    /// 
    ValidationMode GetValidationMode();

    /// <summary> Returns a properties object. Each property corresponds to a
    /// <c>property</c> element in the <c>persistence.xml</c> file.</summary>
    /// <returns> Properties object </returns>
    /// 
    Dictionary<string,string> GetProperties();
    
    /// <summary> Returns the schema version of the <c>persistence.xml</c> file.</summary>
    /// <returns> persistence.xml schema version</returns>
    /// 
    /// 
    string GetPersistenceXMLSchemaVersion();

    /// <summary> Returns ClassLoader that the provider may use to load any
    /// classes, resources, or open URLs.</summary>
    /// <returns> ClassLoader that the provider may use to load any 
    /// classes, resources, or open URLs 
    /// </returns>
    System.Reflection.Assembly GetClassLoader();

    /// <summary> Add a transformer supplied by the provider that will be 
    /// called for every new class definition or class redefinition
    /// that gets loaded by the loader returned by the
    /// <see cref="PersistenceUnitInfo.GetClassLoader"/> method. The transformer 
    /// has no effect on the result returned by the
    /// <see cref="PersistenceUnitInfo.GetNewTempClassLoader"/> method.
    /// Classes are only transformed once within the same classloading
    /// scope, regardless of how many persistence units they may be 
    /// a part of.</summary>
    /// <param name="transformer">   provider-supplied transformer that the
    /// container invokes at class-(re)definition time
    /// </param>
//    void AddTransformer(ClassTransformer transformer);

    /// <summary> Return a new instance of a ClassLoader that the provider may
    /// use to temporarily load any classes, resources, or open
    /// URLs. The scope and classpath of this loader is exactly the
    /// same as that of the loader returned by <see cref="PersistenceUnitInfo.GetClassLoader"/>. None of the classes loaded
    /// by this class loader will be visible to application
    /// components. The provider may only use this ClassLoader within
    /// the scope of the <see cref="PersistenceProvider.CreateContainerEntityManagerFactory"/> call.</summary>
    /// <returns>temporary ClassLoader with same visibility as current loader</returns>
    /// 
    /// 
    System.Reflection.Assembly GetNewTempClassLoader();
}
}